/*
 * Copyright (c) 2011 - 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
*/
#include <gio/gio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>

#include <aul.h>
#include <dlog.h>
#include <glib.h>
#include <db-util.h>

#include "shortcut.h"
#include "shortcut_private.h"
#include "shortcut_db.h"
#include "shortcut_manager.h"
#include "shortcut_internal.h"


#define SHORTCUT_IS_WIDGET_SIZE(size)           (!!((size) & WIDGET_SIZE_DEFAULT))

EAPI int shortcut_set_request_cb(shortcut_request_cb request_cb, void *data)
{
	int ret = _dbus_init();

	if (request_cb == NULL)
		return SHORTCUT_ERROR_INVALID_PARAMETER; /* LCOV_EXCL_LINE */

	if (ret != SHORTCUT_ERROR_NONE) {
		/* LCOV_EXCL_START */
		ErrPrint("Can't init dbus %d", ret);
		return ret;
		/* LCOV_EXCL_STOP */
	}

	ret = _dbus_signal_init();
	if (ret != SHORTCUT_ERROR_NONE) {
		/* LCOV_EXCL_START */
		ErrPrint("Can't init dbus_signal %d", ret);
		return ret;
		/* LCOV_EXCL_STOP */
	}

	ret = _send_service_register();
	if (ret != SHORTCUT_ERROR_NONE) {
		/* LCOV_EXCL_START */
		ErrPrint("Can't init ipc_monitor_register %d", ret);
		return ret;
		/* LCOV_EXCL_STOP */
	}

	ret = _dbus_set_watch_name();
	if (ret != SHORTCUT_ERROR_NONE) {
		/* LCOV_EXCL_START */
		ErrPrint("Can't init _dbus_set_watch_name %d", ret);
		return ret;
		/* LCOV_EXCL_STOP */
	}

	_set_request_cb(request_cb, data);

	return SHORTCUT_ERROR_NONE;
}

EAPI int shortcut_add_to_home(const char *name, shortcut_type type, const char *uri,
		const char *icon, int allow_duplicate, result_cb_t result_cb, void *data)
{
	struct result_cb_item *item;
	char *appid;
	int ret;
	GVariant *body;

	if (ADD_TO_HOME_IS_DYNAMICBOX(type)) {
		/* LCOV_EXCL_START */
		ErrPrint("Invalid type used for adding a shortcut\n");
		return SHORTCUT_ERROR_INVALID_PARAMETER;
		/* LCOV_EXCL_STOP */
	}

	ret = _dbus_init();
	if (ret != SHORTCUT_ERROR_NONE) {
		/* LCOV_EXCL_START */
		ErrPrint("Can't init dbus %d", ret);
		return ret;
		/* LCOV_EXCL_STOP */
	}

	ret = _check_privilege();
	if (ret != SHORTCUT_ERROR_NONE)
		return ret;

	appid = _shortcut_get_pkgname_by_pid();
	item = malloc(sizeof(struct result_cb_item));
	if (!item) {
		/* LCOV_EXCL_START */
		if (appid)
			free(appid);

		ErrPrint("Heap: %d\n", errno);
		return SHORTCUT_ERROR_OUT_OF_MEMORY;
		/* LCOV_EXCL_STOP */
	}

	item->result_cb = result_cb;
	item->result_internal_cb = NULL;
	item->data = data;

	if (!name)
		name = "";

	if (!uri)
		uri = "";

	if (!icon)
		icon = "";

	body = g_variant_new("(ississi)", getpid(), appid, name, type, uri, icon, allow_duplicate);

	ret = _send_async_shortcut(body, item, "add_shortcut");
	if (ret != SHORTCUT_ERROR_NONE) {
		/* LCOV_EXCL_START */
		free(item);
		item = NULL;
		/* LCOV_EXCL_STOP */
	}

	if (appid)
		free(appid);
	if (body)
		g_variant_unref(body);

	return ret;
}

EAPI int shortcut_add_to_home_widget(const char *name, shortcut_widget_size_e size, const char *widget_id,
		const char *icon, double period, int allow_duplicate, result_cb_t result_cb, void *data)
{
	struct result_cb_item *item;
	char *appid;
	int ret;
	GVariant *body;

	if (name == NULL) {
		ErrPrint("AppID is null\n");
		return SHORTCUT_ERROR_INVALID_PARAMETER;
	}

	if (!SHORTCUT_IS_WIDGET_SIZE(size)) {
		/* LCOV_EXCL_START */
		ErrPrint("Invalid type used for adding a widget\n");
		return SHORTCUT_ERROR_INVALID_PARAMETER;
		/* LCOV_EXCL_STOP */
	}

	ret = _dbus_init();
	if (ret != SHORTCUT_ERROR_NONE) {
		/* LCOV_EXCL_START */
		ErrPrint("Can't init dbus %d", ret);
		return ret;
		/* LCOV_EXCL_STOP */
	}

	ret = _check_privilege();
	if (ret != SHORTCUT_ERROR_NONE)
		return ret;

	appid = _shortcut_get_pkgname_by_pid();
	item = malloc(sizeof(struct result_cb_item));
	if (!item) {
		/* LCOV_EXCL_START */
		if (appid)
			free(appid);

		ErrPrint("Heap: %d\n", errno);
		return SHORTCUT_ERROR_OUT_OF_MEMORY;
		/* LCOV_EXCL_STOP */
	}

	item->result_cb = result_cb;
	item->result_internal_cb = NULL;
	item->data = data;

	body = g_variant_new("(ississdi)", getpid(), widget_id, name, size, NULL, icon, period, allow_duplicate);
	ret = _send_async_shortcut(body, item, "add_shortcut_widget");

	if (ret != SHORTCUT_ERROR_NONE) {
		/* LCOV_EXCL_START */
		free(item);
		item = NULL;
		/* LCOV_EXCL_STOP */
	}

	if (appid)
		free(appid);
	if (body)
		g_variant_unref(body);

	return ret;
}

EAPI int shortcut_get_list(const char *package_name, shortcut_list_cb list_cb, void *data)
{
	GDBusMessage *reply = NULL;
	int result;
	int ret = 0;
	GVariant *body;
	GVariant *reply_body;
	GVariant *iter_body;
	GVariantIter *iter;
	GVariantBuilder *b;
	shortcut_info_s shortcut;

	if (list_cb == NULL)
		return SHORTCUT_ERROR_INVALID_PARAMETER;

	result = _dbus_init();
	if (result != SHORTCUT_ERROR_NONE) {
		/* LCOV_EXCL_START */
		ErrPrint("Can't init dbus %d", result);
		return result;
		/* LCOV_EXCL_STOP */
	}

	b = g_variant_builder_new(G_VARIANT_TYPE("a{sv}"));
	if (package_name)
		g_variant_builder_add(b, "{sv}", "package_name", g_variant_new_string(package_name));
	body = g_variant_builder_end(b);
	result = _send_sync_shortcut(g_variant_new("(v)", body), &reply, "get_list");

	if (result == SHORTCUT_ERROR_NONE) {
		reply_body = g_dbus_message_get_body(reply);
		g_variant_get(reply_body, "(ia(v))", &ret, &iter);
		DbgPrint("shortcut count : %d", ret);
		while (g_variant_iter_loop(iter, "(v)", &iter_body)) {
			g_variant_get(iter_body, "(&s&s&s&s&s)",
				&shortcut.package_name, &shortcut.icon, &shortcut.name, &shortcut.extra_key, &shortcut.extra_data);
			DbgPrint("call calback : %s", shortcut.package_name);
			list_cb(shortcut.package_name, shortcut.icon, shortcut.name, shortcut.extra_key, shortcut.extra_data, data);
		}
		g_variant_iter_free(iter);
	} else {
		ret = result; /* LCOV_EXCL_LINE */
	}

	if (reply)
		g_object_unref(reply);

	return ret;
}

/* LCOV_EXCL_START */
EAPI int add_to_home_shortcut(const char *appid, const char *name, int type, const char *content,
		const char *icon, int allow_duplicate, result_internal_cb_t result_cb, void *data)
{
	/*Deprecated API */
	return SHORTCUT_ERROR_NONE;
}
/* LCOV_EXCL_STOP */

/* LCOV_EXCL_START */
EAPI int add_to_home_dynamicbox(const char *appid, const char *name, int type, const char *content, const char *icon, double period, int allow_duplicate, result_internal_cb_t result_cb, void *data)
{
	/*Deprecated API */
	return SHORTCUT_ERROR_NONE;
}
/* LCOV_EXCL_STOP */

/* End of a file */
